#!/usr/bin/python3
"""

    ###################
    ##               ##
    ##    Pi-Clap    ##
    ##               ##
    ###################

Repo: https://github.com/nikhiljohn10/pi-clap
Author: Nikhil John
License: MIT
"""

from array import array

DETECTION_METHODS = [           # Method list for clap detection
    'threshold',
    'filter',
    'fft',
    'wavelet'
]


class SignalProcessor():
    def __init__(self, method):
        self.method_id = DETECTION_METHODS.index(method.name)
        self.method = method

    def findClap(self, data):
        if self.method_id == 1:
            return self.useFiter(data)
        elif self.method_id == 2:
            return self.useFFT(data)
        elif self.method_id == 3:
            return self.useWavelets(data)
        else:
            return self.useThreshold(data)

    def useFiter(self, data):
        # TODO
        return False

    def useFFT(self, data):
        # TODO
        return False

    def useWavelets(self, data):
        # TODO
        return False

    def useThreshold(self, data):
        return (True if max(array('h', data)) > self.method.value else False)



#!/usr/bin/python3
"""

    ###################
    ##               ##
    ##    Pi-Clap    ##
    ##               ##
    ###################

Repo: https://github.com/nikhiljohn10/pi-clap
Author: Nikhil John
License: MIT
"""

from time import sleep
import _thread as thread
import pyaudio

from piclap.settings import Settings
from piclap.processor import SignalProcessor

try:
    import RPi.GPIO
    from .controller import Controller
except(ModuleNotFoundError):
    from .controller import DummyController as Controller
    print("Raspberry Pi GPIO module not installed")

class Listener():
    def __init__(self, config=Settings()):
        self.config = config
        self.input = pyaudio.PyAudio()
        self.stream = self.input.open(format=pyaudio.paInt16,
                                      channels=self.config.channels,
                                      rate=self.config.rate,
                                      input=True,
                                      output=True,
                                      frames_per_buffer=self.config.chunk_size)
        self.claps = 0
        self.exit = False
        self.lock = thread.allocate_lock()
        self.processor = SignalProcessor(method=self.config.method)
        self.rpi = Controller(pin=self.config.pin)

    def clapWait(self, clap):
        sleep(self.config.interval)
        if self.claps > clap:
            self.clapWait(self.claps)

    def listenClaps(self, threadName):
        with self.lock:
            print("Waiting for claps...")
            self.clapWait(self.claps)
            if self.claps == 2:
                self.rpi.flashLight()
            elif self.claps == 3:
                self.rpi.toggleLight(pin=self.config.customPin)
            elif self.claps == 4:
                self.exit = True
            print("You clapped", self.claps, "times.\n")
            self.claps = 0

    def start(self):
        try:
            print("Clap detection started")
            while not self.exit:
                data = self.stream.read(self.config.chunk_size)
                if self.processor.findClap(data):
                    self.claps += 1
                if self.claps == 1 and not self.lock.locked():
                    thread.start_new_thread(
                        self.listenClaps, ("ListenClaps",))
        except(KeyboardInterrupt, SystemExit):
            pass
        self.stop()

    def stop(self):
        print("\rExiting")
        self.stream.stop_stream()
        self.stream.close()
        self.input.terminate()
        self.rpi.cleanup()



from munch import DefaultMunch as Objectify


class Settings():
    def __init__(self):
        self.rate = 44100                   # Number of sample collected in 1sec
        self.channels = 1                   # 1 = Mono Channel
        self.chunk_size = 1024              # Number of frames in the buffer
        self.interval = 0.5                 # Interval between each clap
        self.method = Objectify.fromDict({  # Method used to identify claps
            'name': 'threshold',
            'value': 7000
        })
        self.pin = 24



#!/usr/bin/python3
"""

    ###################
    ##               ##
    ##    Pi-Clap    ##
    ##               ##
    ###################

Repo: https://github.com/nikhiljohn10/pi-clap
Author: Nikhil John
License: MIT
"""
from time import sleep

try:
    import RPi.GPIO as GPIO
except(ModuleNotFoundError):
    pass


class Controller():

    def __init__(self, pin=24):
        self.pin = pin
        GPIO.setmode(GPIO.BCM)
        GPIO.setup(self.pin, GPIO.OUT)

    def flashLight(self, pin=None):
        gpio_pin = pin if pin != None else self.pin
        GPIO.output(gpio_pin, True)
        sleep(1)
        GPIO.output(gpio_pin, False)
        print("Light flashed")

    def toggleLight(self, pin=None):
        gpio_pin = pin if pin != None else self.pin
        GPIO.output(gpio_pin, not GPIO.input(gpio_pin))
        print("Light toggled")

    def cleanup(self):
        GPIO.cleanup()


# This is only used when RPi module is not found in your system
class DummyController():

    def __init__(self, pin=24):
        pass

    def flashLight(self, pin=None):
        print("Light flashed")

    def toggleLight(self, pin=None):
        print("Light toggled")

    def cleanup(self):
        pass
